
#
# $Id: libnet.py,v 1.1.1.1 2001/10/15 06:54:55 davidma Exp $
# 
# Python libnet
# Copyright (C) 2001, David Margrave (davidma@eskimo.com)
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#  


import _libnetc
import string

protocol_headers = ['tcp_options', 'tcp_header', 'udp_header', 'icmp_header']
# some purist will jump on me and say "internetwork" here.  tough
network_headers = ['arp_header', 'ip_header', 'ip_options']
mac_headers = ['eth_header']

def dumphex(s):
    bytes = map(lambda x: '%.2x ' % x, map(ord, s))
    if len(s)>16:
      for i in range((len(s)+16)/16*16, 15, -16):
        bytes.insert(i,'\n')
    print string.join(bytes, '')


class packet:
  def __init__(self):
    self.payload=None
    self.protocol_headers={}
    self.network_headers={}
    self.mac_headers={}

  def do_checksum(self, protocol, pkt_size):
    val = self.getvalue(macheader=0)
    ip_hdrlen = (ord(val[0]) & 0x0f) * 4
    _libnetc.do_checksum(val, protocol, pkt_size)
    if protocol==_libnetc.IPPROTO_TCP:
      self.protocol_headers['tcp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]
    if protocol==_libnetc.IPPROTO_IP:
      self.network_headers['ip_header'] = val[0:ip_hdrlen]
    elif protocol==_libnetc.IPPROTO_ICMP:
      self.protocol_headers['icmp_header'] = val[ip_hdrlen:]
    if protocol==_libnetc.IPPROTO_TCP:
      self.protocol_headers['tcp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]
    if protocol==_libnetc.IPPROTO_UDP:
      self.protocol_headers['udp_header'] = val[ip_hdrlen:ip_hdrlen+pkt_size]

  def getvalue(self, macheader=1):
    value = ''
    if macheader:
      for val in mac_headers:
        if self.mac_headers.has_key(val):
          value = self.mac_headers[val]
          break
    for val in network_headers:
      if self.network_headers.has_key(val):
        value = value + self.network_headers[val]
        break
    for val in protocol_headers:
      if self.protocol_headers.has_key(val):
        value = value + self.protocol_headers[val]
        break
    if self.payload:
      value = value + self.payload
    return value

  def __repr__(self):
    return self.getvalue()

  def __len__(self):
    return len(self.__repr__())
     
     
  def build_ethernet(self, *args, **kwargs):
    self.mac_headers['eth_header'] = apply(_libnetc.build_ethernet, args, kwargs)

  def build_arp(self, *args, **kwargs):
    self.network_headers['arp_header'] = apply(_libnetc.build_arp, args, kwargs)

  def build_ip(self, *args, **kwargs):
    self.network_headers['ip_header'] = apply(_libnetc.build_ip, args, kwargs)

  def build_icmp_echo(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_echo, args, kwargs)

  def build_icmp_mask(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_mask, args, kwargs)

  def build_icmp_unreach(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_unreach, args, kwargs)

  def build_icmp_timeexceeded(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_timeexceeded, args, kwargs)

  def build_icmp_redirect(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_redirect, args, kwargs)

  def build_icmp_timestamp(self, *args, **kwargs):
    self.protocol_headers['icmp_header'] = apply(_libnetc.build_icmp_timestamp, args, kwargs)

  def build_tcp(self, *args, **kwargs):
    self.protocol_headers['tcp_header'] = apply(_libnetc.build_tcp, args, kwargs)
    
  def build_udp(self, *args, **kwargs):
    self.protocol_headers['udp_header'] = apply(_libnetc.build_udp, args, kwargs)

  def build_dns(self, *args, **kwargs):
    self.payload = apply(_libnetc.build_dns, args, kwargs)
    


class interfacePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self):
        if self.thisown == 1 :
            _libnetc.delete_interface(self.this)
    def open_link(self,arg0):
        val = _libnetc.interface_open_link(self.this,arg0)
        return val
    def open_raw(self,arg0):
        val = _libnetc.interface_open_raw(self.this,arg0)
        return val
    def write(self,arg0):
        if isinstance(arg0, packet):
          arg0=repr(arg0)
        val = _libnetc.interface_write(self.this,arg0)
        return val
    def get_ipaddr(self):
        val = _libnetc.interface_get_ipaddr(self.this)
        return val
    def get_hwaddr(self):
        val = _libnetc.interface_get_hwaddr(self.this)
        return val
    def __setattr__(self,name,value):
        if name == "link" :
            _libnetc.interface_link_set(self.this,value)
            return
        if name == "fd" :
            _libnetc.interface_fd_set(self.this,value)
            return
        if name == "device" :
            _libnetc.interface_device_set(self.this,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "link" : 
            return _libnetc.interface_link_get(self.this)
        if name == "fd" : 
            return _libnetc.interface_fd_get(self.this)
        if name == "device" : 
            return _libnetc.interface_device_get(self.this)
        raise AttributeError,name
    def __repr__(self):
        return "<C interface instance>"
class interface(interfacePtr):
    def __init__(self) :
        self.this = _libnetc.new_interface()
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------

build_arp = _libnetc.build_arp

build_dns = _libnetc.build_dns

build_ethernet = _libnetc.build_ethernet

build_icmp_echo = _libnetc.build_icmp_echo

build_icmp_mask = _libnetc.build_icmp_mask

build_icmp_unreach = _libnetc.build_icmp_unreach

build_icmp_timeexceed = _libnetc.build_icmp_timeexceed

build_icmp_timestamp = _libnetc.build_icmp_timestamp

build_icmp_redirect = _libnetc.build_icmp_redirect

build_ip = _libnetc.build_ip

build_tcp = _libnetc.build_tcp

build_udp = _libnetc.build_udp

select_device = _libnetc.select_device

host_lookup = _libnetc.host_lookup

name_resolve = _libnetc.name_resolve



#-------------- VARIABLE WRAPPERS ------------------

IPPROTO_IP = _libnetc.IPPROTO_IP
IPPROTO_TCP = _libnetc.IPPROTO_TCP
IPPROTO_UDP = _libnetc.IPPROTO_UDP
IPPROTO_ICMP = _libnetc.IPPROTO_ICMP
IPPROTO_IGMP = _libnetc.IPPROTO_IGMP
IPPROTO_OSPF = _libnetc.IPPROTO_OSPF
IPPROTO_OSPF_LSA = _libnetc.IPPROTO_OSPF_LSA
IPPROTO_VRRP = _libnetc.IPPROTO_VRRP
IPPROTO_RAW = _libnetc.IPPROTO_RAW
ARP_H = _libnetc.ARP_H
DNS_H = _libnetc.DNS_H
ETH_H = _libnetc.ETH_H
ICMP_H = _libnetc.ICMP_H
ICMP_ECHO_H = _libnetc.ICMP_ECHO_H
ICMP_MASK_H = _libnetc.ICMP_MASK_H
ICMP_UNREACH_H = _libnetc.ICMP_UNREACH_H
ICMP_TIMXCEED_H = _libnetc.ICMP_TIMXCEED_H
ICMP_REDIRECT_H = _libnetc.ICMP_REDIRECT_H
ICMP_TS_H = _libnetc.ICMP_TS_H
IGMP_H = _libnetc.IGMP_H
IP_H = _libnetc.IP_H
RIP_H = _libnetc.RIP_H
TCP_H = _libnetc.TCP_H
UDP_H = _libnetc.UDP_H
VRRP_H = _libnetc.VRRP_H
IP_MAXPACKET = _libnetc.IP_MAXPACKET
MAX_IPOPTLEN = _libnetc.MAX_IPOPTLEN
IP_OFFMASK = _libnetc.IP_OFFMASK
IP_DF = _libnetc.IP_DF
IP_MF = _libnetc.IP_MF
TH_URG = _libnetc.TH_URG
TH_ACK = _libnetc.TH_ACK
TH_PUSH = _libnetc.TH_PUSH
TH_RST = _libnetc.TH_RST
TH_SYN = _libnetc.TH_SYN
TH_FIN = _libnetc.TH_FIN
ICMP_ECHOREPLY = _libnetc.ICMP_ECHOREPLY
ICMP_UNREACH = _libnetc.ICMP_UNREACH
ICMP_SOURCEQUENCH = _libnetc.ICMP_SOURCEQUENCH
ICMP_REDIRECT = _libnetc.ICMP_REDIRECT
ICMP_ECHO = _libnetc.ICMP_ECHO
ICMP_ROUTERADVERT = _libnetc.ICMP_ROUTERADVERT
ICMP_ROUTERSOLICIT = _libnetc.ICMP_ROUTERSOLICIT
ICMP_TIMXCEED = _libnetc.ICMP_TIMXCEED
ICMP_PARAMPROB = _libnetc.ICMP_PARAMPROB
ICMP_TSTAMP = _libnetc.ICMP_TSTAMP
ICMP_TSTAMPREPLY = _libnetc.ICMP_TSTAMPREPLY
ICMP_IREQ = _libnetc.ICMP_IREQ
ICMP_IREQREPLY = _libnetc.ICMP_IREQREPLY
ICMP_MASKREQ = _libnetc.ICMP_MASKREQ
ICMP_MASKREPLY = _libnetc.ICMP_MASKREPLY
ICMP_UNREACH_NET = _libnetc.ICMP_UNREACH_NET
ICMP_UNREACH_HOST = _libnetc.ICMP_UNREACH_HOST
ICMP_UNREACH_PROTOCOL = _libnetc.ICMP_UNREACH_PROTOCOL
ICMP_UNREACH_PORT = _libnetc.ICMP_UNREACH_PORT
ICMP_UNREACH_NEEDFRAG = _libnetc.ICMP_UNREACH_NEEDFRAG
ICMP_UNREACH_SRCFAIL = _libnetc.ICMP_UNREACH_SRCFAIL
ICMP_UNREACH_NET_UNKNOWN = _libnetc.ICMP_UNREACH_NET_UNKNOWN
ICMP_UNREACH_HOST_UNKNOWN = _libnetc.ICMP_UNREACH_HOST_UNKNOWN
ICMP_UNREACH_ISOLATED = _libnetc.ICMP_UNREACH_ISOLATED
ICMP_UNREACH_NET_PROHIB = _libnetc.ICMP_UNREACH_NET_PROHIB
ICMP_UNREACH_HOST_PROHIB = _libnetc.ICMP_UNREACH_HOST_PROHIB
ICMP_UNREACH_TOSNET = _libnetc.ICMP_UNREACH_TOSNET
ICMP_UNREACH_TOSHOST = _libnetc.ICMP_UNREACH_TOSHOST
ICMP_UNREACH_FILTER_PROHIB = _libnetc.ICMP_UNREACH_FILTER_PROHIB
ICMP_UNREACH_HOST_PRECEDENCE = _libnetc.ICMP_UNREACH_HOST_PRECEDENCE
ICMP_UNREACH_PRECEDENCE_CUTOFF = _libnetc.ICMP_UNREACH_PRECEDENCE_CUTOFF
ICMP_REDIRECT_NET = _libnetc.ICMP_REDIRECT_NET
ICMP_REDIRECT_HOST = _libnetc.ICMP_REDIRECT_HOST
ICMP_REDIRECT_TOSNET = _libnetc.ICMP_REDIRECT_TOSNET
ICMP_REDIRECT_TOSHOST = _libnetc.ICMP_REDIRECT_TOSHOST
ICMP_TIMXCEED_INTRANS = _libnetc.ICMP_TIMXCEED_INTRANS
ICMP_TIMXCEED_REASS = _libnetc.ICMP_TIMXCEED_REASS
ICMP_PARAMPROB_OPTABSENT = _libnetc.ICMP_PARAMPROB_OPTABSENT
IGMP_MEMBERSHIP_QUERY = _libnetc.IGMP_MEMBERSHIP_QUERY
IGMP_V1_MEMBERSHIP_REPORT = _libnetc.IGMP_V1_MEMBERSHIP_REPORT
IGMP_V2_MEMBERSHIP_REPORT = _libnetc.IGMP_V2_MEMBERSHIP_REPORT
IGMP_LEAVE_GROUP = _libnetc.IGMP_LEAVE_GROUP
ARPHRD_ETHER = _libnetc.ARPHRD_ETHER
ARPOP_REQUEST = _libnetc.ARPOP_REQUEST
ARPOP_REPLY = _libnetc.ARPOP_REPLY
ARPOP_REVREQUEST = _libnetc.ARPOP_REVREQUEST
ARPOP_REVREPLY = _libnetc.ARPOP_REVREPLY
ARPOP_INVREQUEST = _libnetc.ARPOP_INVREQUEST
ARPOP_INVREPLY = _libnetc.ARPOP_INVREPLY
RIPCMD_REQUEST = _libnetc.RIPCMD_REQUEST
RIPCMD_RESPONSE = _libnetc.RIPCMD_RESPONSE
RIPCMD_TRACEON = _libnetc.RIPCMD_TRACEON
RIPCMD_TRACEOFF = _libnetc.RIPCMD_TRACEOFF
RIPCMD_POLL = _libnetc.RIPCMD_POLL
RIPCMD_POLLENTRY = _libnetc.RIPCMD_POLLENTRY
RIPCMD_MAX = _libnetc.RIPCMD_MAX
RIPVER_0 = _libnetc.RIPVER_0
RIPVER_1 = _libnetc.RIPVER_1
RIPVER_2 = _libnetc.RIPVER_2
OSPF_H = _libnetc.OSPF_H
HELLO_H = _libnetc.HELLO_H
DBD_H = _libnetc.DBD_H
LSR_H = _libnetc.LSR_H
LSU_H = _libnetc.LSU_H
LSA_H = _libnetc.LSA_H
AUTH_H = _libnetc.AUTH_H
OSPF_CKSUM = _libnetc.OSPF_CKSUM
LS_RTR_LEN = _libnetc.LS_RTR_LEN
LS_NET_LEN = _libnetc.LS_NET_LEN
LS_SUM_LEN = _libnetc.LS_SUM_LEN
LS_AS_EXT_LEN = _libnetc.LS_AS_EXT_LEN
OSPF_UMD = _libnetc.OSPF_UMD
OSPF_HELLO = _libnetc.OSPF_HELLO
OSPF_DBD = _libnetc.OSPF_DBD
OSPF_LSR = _libnetc.OSPF_LSR
OSPF_LSU = _libnetc.OSPF_LSU
OSPF_LSA = _libnetc.OSPF_LSA
OSPF_AUTH_NULL = _libnetc.OSPF_AUTH_NULL
OSPF_AUTH_SIMPLE = _libnetc.OSPF_AUTH_SIMPLE
OSPF_AUTH_MD5 = _libnetc.OSPF_AUTH_MD5
OPT_EBIT = _libnetc.OPT_EBIT
OPT_MCBIT = _libnetc.OPT_MCBIT
OPT_NPBIT = _libnetc.OPT_NPBIT
OPT_EABIT = _libnetc.OPT_EABIT
OPT_DCBIT = _libnetc.OPT_DCBIT
DBD_IBIT = _libnetc.DBD_IBIT
DBD_MBIT = _libnetc.DBD_MBIT
DBD_MSBIT = _libnetc.DBD_MSBIT
LS_TYPE_RTR = _libnetc.LS_TYPE_RTR
LS_TYPE_NET = _libnetc.LS_TYPE_NET
LS_TYPE_IP = _libnetc.LS_TYPE_IP
LS_TYPE_ASBR = _libnetc.LS_TYPE_ASBR
LS_TYPE_ASEXT = _libnetc.LS_TYPE_ASEXT
RTR_FLAGS_W = _libnetc.RTR_FLAGS_W
RTR_FLAGS_E = _libnetc.RTR_FLAGS_E
RTR_FLAGS_B = _libnetc.RTR_FLAGS_B
LINK_ID_NBR_ID = _libnetc.LINK_ID_NBR_ID
LINK_ID_IP_DES = _libnetc.LINK_ID_IP_DES
LINK_ID_SUB = _libnetc.LINK_ID_SUB
RTR_TYPE_PTP = _libnetc.RTR_TYPE_PTP
RTR_TYPE_TRANS = _libnetc.RTR_TYPE_TRANS
RTR_TYPE_STUB = _libnetc.RTR_TYPE_STUB
AS_E_BIT_ON = _libnetc.AS_E_BIT_ON
VRRP_TYPE_ADVERT = _libnetc.VRRP_TYPE_ADVERT
VRRP_AUTH_NONE = _libnetc.VRRP_AUTH_NONE
VRRP_AUTH_PASSWD = _libnetc.VRRP_AUTH_PASSWD
VRRP_AUTH_IPAH = _libnetc.VRRP_AUTH_IPAH
PRAND_MAX = _libnetc.PRAND_MAX
PR2 = _libnetc.PR2
PR8 = _libnetc.PR8
PR16 = _libnetc.PR16
PRu16 = _libnetc.PRu16
PR32 = _libnetc.PR32
PRu32 = _libnetc.PRu32
ERR_WARNING = _libnetc.ERR_WARNING
ERR_CRITICAL = _libnetc.ERR_CRITICAL
ERR_FATAL = _libnetc.ERR_FATAL
DONT_RESOLVE = _libnetc.DONT_RESOLVE
RESOLVE = _libnetc.RESOLVE
ETHER_ADDR_LEN = _libnetc.ETHER_ADDR_LEN
ETHERTYPE_PUP = _libnetc.ETHERTYPE_PUP
ETHERTYPE_IP = _libnetc.ETHERTYPE_IP
ETHERTYPE_ARP = _libnetc.ETHERTYPE_ARP
ETHERTYPE_REVARP = _libnetc.ETHERTYPE_REVARP
ETHERTYPE_VLAN = _libnetc.ETHERTYPE_VLAN
ETHERTYPE_LOOPBACK = _libnetc.ETHERTYPE_LOOPBACK
